clear all; clc;
%System parameters
nSamp=5; %Number of samples per symbol determines baud rate Tsym
Fs=100; %Sampling Frequency of the system
Ts=1/Fs; %Sampling period
Tsym=nSamp*Ts;

%Define transfer function of the channel
k=6; %define limits for computing channel response
N0 = 0.1; %Standard deviation of AWGN noise
t = -k*Tsym:Ts:k*Tsym; %time base defined till +/-kTsym
h = 1./(1+(t/Tsym).^2); %channel model, replace with your own model
h = h + N0*randn(1,length(h)); %add Noise to the channel response
h_c= h(1:nSamp:end);%downsampling to represent symbol rate sampler
t_inst=t(1:nSamp:end); %symbol sampling instants

figure;plot(t,h);hold on;%channel response at all sampling instants
stem(t_inst,h_c,'r'); %channel response at symbol sampling instants
legend('channel response','at symbol samp instants');
title('Channel Model - Impulse response'); 
xlabel('Time (s)');ylabel('Amplitude');

%Equalizer Design Parameters
nTaps = 14; %Desired number of taps for equalizer filter

%design DELAY OPTIMIZED MMSE eq. for given channel, get tap weights
%and filter the input through the equalizer
noiseVariance = N0^2; %noise variance
snr = 10*log10(1/N0); %convert to SNR (assume var(signal) = 1)
[h_eq,MSE,optDelay]=mmse_equalizer(h_c,snr,nTaps); %find eq. co-effs

xn=h_c; %Test the equalizer with the channel response as input
yn=conv(h_eq,xn); %filter input through the equalizer
h_sys=conv(h_eq,h_c); %overall effect of channel and equalizer

disp(['MMSE Equalizer Design: N=', num2str(nTaps), ...
    ' Delay=',num2str(optDelay)])
disp('MMSE equalizer weights:'); disp(h_eq)

figure; subplot(1,2,1); stem(0:1:length(xn)-1,xn);
xlabel('Samples'); ylabel('Amplitude');title('Equalizer input');

subplot(1,2,2); stem(0:1:length(yn)-1,yn);
xlabel('Samples'); ylabel('Amplitude');
title(['Equalizer output- N=', num2str(nTaps),' Delay=',num2str(optDelay)]);