function [PL,Pr_dBm] = FriisModel(Pt_dBm,Gt_dBi,Gr_dBi,f,d,L,n)
%Pt_dBm = Transmitted power in dBm
%Gt_dBi = Gain of the Transmitted antenna in dBi
%Gr_dBi = Gain of the Receiver antenna in dBi
%f = frequency of transmitted signal in Hertz
%d = array of distances at which the loss needs to be calculated
%L = Other System Losses, No Loss case L=1
%n = path loss exponent (n=2 for free space)
%Pr_dBm = Received power in dBm
%PL = path loss due to log normal shadowing
lambda=3*10^8/f; %Wavelength in meters
K = 20*log10(lambda/(4*pi))-10*n*log10(d)-10*log10(L);%constant path-loss factor
PL = Gt_dBi + Gr_dBi + K; %PL(d) - Path loss at d meters
Pr_dBm = Pt_dBm + PL; %Receieved power in dBm at d meters
end